import pytest

from torah_dl.core.exceptions import NetworkError


class TestYutorahExtractor:
    from torah_dl.core.extractors import YutorahExtractor

    extractor = YutorahExtractor()

    testdata = [
        pytest.param(
            "https://www.yutorah.org/lectures/1116616/Praying-for-Rain-and-the-International-Traveler",
            "https://download.yutorah.org/2024/986/1116616/praying-for-rain-and-the-international-traveler.mp3",
            "Praying for Rain and the International Traveler",
            "mp3",
            id="main_page",
        ),
        pytest.param(
            "https://www.yutorah.org/lectures/1117459/",
            "https://download.yutorah.org/2024/986/1117459/davening-with-strep-throat.mp3",
            "Davening with Strep Throat",
            "mp3",
            id="short_link",
        ),
        pytest.param(
            "https://www.yutorah.org/lectures/details?shiurid=1117409",
            "https://download.yutorah.org/2024/21197/1117409/ketubot-42-dechitat-aveilut-1.mp3",
            "Ketubot 42: Dechitat Aveilut (1)",
            "mp3",
            id="shiurid_link",
        ),
    ]

    @pytest.mark.parametrize("url, download_url, title, file_format", testdata)
    def test_can_handle(self, url: str, download_url: str, title: str, file_format: str):
        assert self.extractor.can_handle(url)

    @pytest.mark.parametrize("url, download_url, title, file_format", testdata)
    def test_extract(self, url: str, download_url: str, title: str, file_format: str):
        result = self.extractor.extract(url)
        assert result.download_url == download_url
        assert result.title == title
        assert result.file_format == file_format

    def test_extract_invalid_link(self):
        with pytest.raises(NetworkError):
            self.extractor.extract("https://www.yutorah.org/lectures/details?shiurid=0000000/")



import pytest

from torah_dl import extract
from torah_dl.core.exceptions import ExtractorNotFoundError


def test_extract():
    url = "https://www.yutorah.org/lectures/1116616/Praying-for-Rain-and-the-International-Traveler"
    download_url = "https://download.yutorah.org/2024/986/1116616/praying-for-rain-and-the-international-traveler.mp3"

    title = "Praying for Rain and the International Traveler"
    file_format = "mp3"

    extraction = extract(url)
    assert extraction.download_url == download_url
    assert extraction.title == title
    assert extraction.file_format == file_format


def test_extract_failed():
    with pytest.raises(ExtractorNotFoundError):
        extract("https://www.gashmius.xyz/")



from .exceptions import ExtractorNotFoundError
from .extractors import YutorahExtractor
from .models import Extraction

EXTRACTORS = [YutorahExtractor()]


def extract(url: str) -> Extraction:
    for extractor in EXTRACTORS:
        if extractor.can_handle(url):
            return extractor.extract(url)

    raise ExtractorNotFoundError(url)



import importlib.metadata
from pathlib import Path
from typing import Annotated

import typer
from rich.console import Console
from rich.table import Table

from torah_dl import download, extract
from torah_dl.core.exceptions import ExtractorNotFoundError

try:
    __version__ = importlib.metadata.version(__package__ or __name__)
except importlib.metadata.PackageNotFoundError:
    __version__ = "develop"

app = typer.Typer()
console = Console()


@app.command(name="extract")
def extract_url(
    url: str,
    url_only: Annotated[bool, typer.Option("--url-only", help="Only output the download URL")] = False,
):
    """
    Extract information from a given URL
    """
    try:
        extraction = extract(url)
    except ExtractorNotFoundError:
        typer.echo(f"Extractor not found for URL: {url}", err=True)
        raise typer.Exit(1) from None

    table = Table(box=None, pad_edge=False)
    table.add_column("Title", style="cyan")
    table.add_column("Download URL", style="green")
    table.add_row(extraction.title, extraction.download_url)
    if url_only:
        typer.echo(extraction.download_url)
    else:
        console.print(table)


@app.command(name="download")
def download_url(
    url: Annotated[str, typer.Argument(help="URL to download")],
    output_path: Annotated[Path, typer.Argument(help="Path to save the downloaded file")] = Path("audio"),
):
    """Download a file from a URL and show progress."""
    with console.status("Extracting URL..."):
        extraction = extract(url)
    with console.status("Downloading file..."):
        download(extraction.download_url, output_path)


def version_callback(value: bool):
    """
    print version information to shell
    """
    if value:
        typer.echo(f"torah-dl version: {__version__}")
        raise typer.Exit()


@app.callback(invoke_without_command=True)
def callback(
    version: Annotated[
        bool | None,
        typer.Option("--version", callback=version_callback, is_eager=True),
    ] = None,
):
    """
    TODO: add description
    """
    pass


if __name__ == "__main__":
    app()
